@@ -28,7 +28,7 @@ import os
 import string, tempfile, random, cStringIO, os
 
 import urlgrabber.grabber
-from urlgrabber.grabber import URLGrabber, URLGrabError
+from urlgrabber.grabber import URLGrabber, URLGrabError, URLGrabberOptions
 import urlgrabber.mirror
 from urlgrabber.mirror import MirrorGroup, MGRandomStart, MGRandomOrder
 
@@ -106,6 +106,9 @@ class CallbackTests(TestCase):
         self.g  = URLGrabber()
         fullmirrors = [base_mirror_url + m + '/' for m in \
                        (bad_mirrors + good_mirrors)]
+        if hasattr(urlgrabber.grabber, '_TH'):
+            # test assumes mirrors are not re-ordered
+            urlgrabber.grabber._TH.hosts.clear()
         self.mg = MirrorGroup(self.g, fullmirrors)
     
     def test_failure_callback(self):
@@ -168,6 +171,7 @@ class FakeGrabber:
         self.resultlist = resultlist or []
         self.index = 0
         self.calls = []
+        self.opts = URLGrabberOptions()
         
     def urlgrab(self, url, filename=None, **kwargs):
         self.calls.append( (url, filename) )
@@ -264,6 +268,73 @@ class ActionTests(TestCase):
         self.assertEquals(self.g.calls, expected_calls)
         self.assertEquals(urlgrabber.mirror.DEBUG.logs, expected_logs)
                 
+import thread, socket
+LOCALPORT = 'localhost', 2000
+
+class HttpReplyCode(TestCase):
+    def setUp(self):
+        # start the server
+        self.exit = False
+        def server():
+            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+            s.bind(LOCALPORT); s.listen(1)
+            while 1:
+                c, a = s.accept()
+                if self.exit: c.close(); break
+                while not c.recv(4096).endswith('\r\n\r\n'): pass
+                c.sendall('HTTP/1.1 %d %s\r\n' % self.reply)
+                if self.content is not None:
+                    c.sendall('Content-Length: %d\r\n\r\n' % len(self.content))
+                    c.sendall(self.content)
+                c.close()
+            s.close()
+            self.exit = False
+        thread.start_new_thread(server, ())
+
+        # create grabber and mirror group objects
+        def failure(obj):
+            self.code = getattr(obj.exception, 'code', None)
+            return {}
+        self.g  = URLGrabber()
+        self.mg = MirrorGroup(self.g, ['http://%s:%d' % LOCALPORT],
+                              failure_callback = failure)
+
+    def tearDown(self):
+        # shut down the server
+        self.exit = True
+        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        s.connect(LOCALPORT); s.close() # wake it up
+        while self.exit: pass # poor man's join
+
+    def test_grab(self):
+        'tests the propagation of HTTP reply code'
+        self.reply = 503, "Busy"
+        self.content = None
+
+        # single
+        self.assertRaises(URLGrabError, self.mg.urlgrab, 'foo')
+        self.assertEquals(self.code, 503); del self.code
+
+        # multi
+        err = []
+        self.mg.urlgrab('foo', async = True, failfunc = err.append)
+        urlgrabber.grabber.parallel_wait()
+        self.assertEquals([e.exception.errno for e in err], [256])
+        self.assertEquals(self.code, 503); del self.code
+
+    def test_range(self):
+        'test client-side processing of HTTP ranges'
+        # server does not process ranges
+        self.reply = 200, "OK"
+        self.content = 'ABCDEF'
+
+        # no range specified
+        data = self.mg.urlread('foo')
+        self.assertEquals(data, 'ABCDEF')
+
+        data = self.mg.urlread('foo', range = (3, 5))
+        self.assertEquals(data, 'DE')
 
 def suite():
     tl = TestLoader()
