@@ -13,8 +13,8 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 # Copyright 2002-2006 Michael D. Stenner, Ryan Tomayko
+# Copyright 2009 Red Hat, Inc - pycurl support added by Seth Vidal
 
-# $Id: __init__.py,v 1.17 2006/03/02 21:06:35 mstenner Exp $
 
 """A high-level cross-protocol url-grabber.
 
@@ -44,10 +44,12 @@ following features:
     automatically switching mirrors if there is a failure.
 """
 
-__version__ = '2.9.9'
-__date__    = '2006/03/02'
+__version__ = '3.10.1'
+__date__    = '2013/12/18'
 __author__  = 'Michael D. Stenner <mstenner@linux.duke.edu>, ' \
-              'Ryan Tomayko <rtomayko@naeblis.cx>'
-__url__     = 'http://linux.duke.edu/projects/urlgrabber/'
+              'Ryan Tomayko <rtomayko@naeblis.cx>' \
+              'Seth Vidal <skvidal@fedoraproject.org>' \
+              'Zdenek Pavlas <zpavlas@redhat.com>'
+__url__     = 'http://urlgrabber.baseurl.org/'
 
 from grabber import urlgrab, urlopen, urlread
