@@ -999,7 +999,7 @@ class URLGrabberOptions:
         self.max_connections = 5
         self.timedhosts = None
         self.half_life = 30*24*60*60 # 30 days
-        self.default_speed = 1e6 # 1 MBit
+        self.default_speed = 500e3 # 500 kBps
         self.ftp_disable_epsv = False
         
     def __repr__(self):
@@ -1249,6 +1249,7 @@ class PyCurlFileObject(object):
         self._tsize = 0
         self._amount_read = 0
         self._reget_length = 0
+        self._range = None
         self._prog_running = False
         self._error = (None, None)
         self.size = 0
@@ -1288,7 +1289,15 @@ class PyCurlFileObject(object):
 
             self._amount_read += len(buf)
             try:
-                self.fo.write(buf)
+                if self._range:
+                    # client-side ranges
+                    pos = self._amount_read - len(buf)
+                    start = self._range[0] - pos
+                    stop = self._range[1] - pos
+                    if start < len(buf) and stop > 0:
+                        self.fo.write(buf[max(start, 0):stop])
+                else:
+                    self.fo.write(buf)
             except IOError, e:
                 self._cb_error = URLGrabError(16, exception2msg(e))
                 return -1
@@ -1313,13 +1322,14 @@ class PyCurlFileObject(object):
                 if buf.lower().find('content-length:') != -1:
                     length = buf.split(':')[1]
                     self.size = int(length)
-                elif self.append and self._hdr_dump == '' and ' 200 ' in buf:
+                elif (self.append or self.opts.range) and self._hdr_dump == '' and ' 200 ' in buf:
                     # reget was attempted but server sends it all
                     # undo what we did in _build_range()
                     self.append = False
                     self.reget_time = None
                     self._amount_read = 0
                     self._reget_length = 0
+                    self._range = self.opts.range
                     self.fo.truncate(0)
             elif self.scheme in ['ftp']:
                 s = None
@@ -1893,6 +1903,9 @@ class PyCurlFileObject(object):
             urllib.addinfourl, via. urllib.URLopener.* """
         return self.url
         
+if hasattr(pycurl, 'GLOBAL_ACK_EINTR'):
+    # fail immediately on ctrl-c
+    pycurl.global_init(pycurl.GLOBAL_DEFAULT | pycurl.GLOBAL_ACK_EINTR)
 _curl_cache = pycurl.Curl() # make one and reuse it over and over and over
 
 def reset_curl_obj():
@@ -2170,6 +2183,7 @@ def parallel_wait(meter=None):
     dl = _ExternalDownloaderPool()
     host_con = {} # current host connection counts
     single = set() # hosts in single connection mode
+    retry_queue = []
 
     def start(opts, tries):
         opts.tries = tries
@@ -2221,6 +2235,9 @@ def parallel_wait(meter=None):
                 # Turn on the max_connections=1 override. BZ 853432
                 if DEBUG: DEBUG.info('max_connections(%s) %s => 1', key, limit)
                 single.add(key)
+                # When using multi-downloader the parent's _curl_cache
+                # object is idle. Kill it, as it might use keepalive=1.
+                reset_curl_obj()
 
             retry = opts.retry or 0
             if opts.failure_callback:
@@ -2247,7 +2264,7 @@ def parallel_wait(meter=None):
                     # mask this mirror and retry
                     if action.get('remove', 1):
                         removed.add(key)
-                    _async_queue.append(opts)
+                    retry_queue.append(opts)
                     continue
                 # fail=1 from callback
                 ug_err.errors = errors
@@ -2257,19 +2274,22 @@ def parallel_wait(meter=None):
             _run_callback(opts.failfunc, opts)
 
     try:
-        idx = 0
+        retry_idx = idx = 0
         while True:
-            if idx >= len(_async_queue):
-                # the queue is empty
+            if retry_idx < len(retry_queue):
+                # retries first
+                opts = retry_queue[retry_idx]
+                retry_idx += 1
+            elif idx < len(_async_queue):
+                # handle next request
+                opts = _async_queue[idx]
+                idx += 1
+            else:
+                # both queues are empty
                 if not dl.running: break
-                # pending dl may extend it
                 perform()
                 continue
 
-            # handle next request
-            opts = _async_queue[idx]
-            idx += 1
-
             # check global limit
             while len(dl.running) >= default_grabber.opts.max_connections:
                 perform()
